//
// QtTUIOPaintDemo
//
// qpaintwidget.h
// Copyright 2013 Touch-Base Ltd. All Rights Reserved
//

#ifndef QPAINTWIDGET_H
#define QPAINTWIDGET_H

#include <QWidget>
#include <QEvent>
#include <QList>
#include <QMap>
#include <QColor>
#include <QElapsedTimer>
#include <QTimer>

class PaintBrush {
public:
	PaintBrush() : lastPosition(), index(0), down(false) {};

	QPoint	lastPosition;
	int		index;
	bool	down;
};

#define QCustomTouchEventType QEvent::Type(QEvent::User + 100)

class QCustomTouchEvent : public QEvent
{
public:
	QCustomTouchEvent() : QEvent(QCustomTouchEventType) {};

	Qt::TouchPointState		state;
	int						index;
	QPoint					position;
	QPointF					normalizedPosition;
};


class QPaintWidget : public QWidget
{
	Q_OBJECT
public:
    explicit QPaintWidget(QWidget *parent = 0);

	void addTouch(int index, QPointF normalizedPosition);
	void updateTouch(int index, QPointF normalizedPosition);
    void removeTouch(int index);
	void setTouchLabel(const QString &newLabel);

protected:
	void resizeImage(QImage *image, const QSize &newSize);
	void resizeEvent(QResizeEvent *event);
	void mousePressEvent(QMouseEvent *mouseEvent);
	void mouseDoubleClickEvent(QMouseEvent *mouseEvent);
	void mouseMoveEvent(QMouseEvent *mouseEvent);
	void mouseReleaseEvent(QMouseEvent *mouseEvent);
	void paintEvent(QPaintEvent *event);
    bool event(QEvent *event);

	void drawText(QPainter &painter, QString &text, int x, int y);
	void drawBrushLabel(QPainter &painter, const PaintBrush &brush, const QString &label);
	void paintLine(QPoint startPos, QPoint endPos, int radius, QColor color);

	void postTouchEvent(Qt::TouchPointState type, int index, QPointF normalizedPosition);
	void startBrush(PaintBrush &brush, QPoint position, int index, const QList<QColor> &colors);
	void moveBrush(PaintBrush &brush, QPoint position, const QList<QColor> &colors);
	void removeBrush(PaintBrush &brush, QPoint finalPosition, const QList<QColor> &colors);

signals:

public slots:

private:
	QList<QColor>			mouseColors;
	QList<QColor>			touchColors;

	QImage					image;
	PaintBrush				mouseBrush;
	QMap<int, PaintBrush>	touchBrushes;
	QString					touchLabel;
};

#endif // QPAINTWIDGET_H
